import typing
import collections.abc
import typing_extensions
from . import complete_calltip as complete_calltip
from . import complete_import as complete_import
from . import complete_namespace as complete_namespace
from . import intellisense as intellisense

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")



import typing
import collections.abc
import typing_extensions
import bpy.types

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

class ObjectModeOperator:
    @classmethod
    def poll(cls, context):
        """

        :param context:
        """
        ...

class QuickExplode(ObjectModeOperator, bpy.types.Operator):
    """Make selected objects explode"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

    def invoke(self, context, _event):
        """

        :param context:
        :param _event:
        """
        ...

class QuickFur(ObjectModeOperator, bpy.types.Operator):
    """Add a fur setup to the selected objects"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

class QuickLiquid(bpy.types.Operator):
    """Make selected objects liquid"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

class QuickSmoke(ObjectModeOperator, bpy.types.Operator):
    """Use selected objects as smoke emitters"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

def grid_location(x, y): ...
def obj_bb_minmax(obj, min_co, max_co): ...
def object_ensure_material(obj, mat_name):
    """Use an existing material or add a new one."""

    ...



import typing
import collections.abc
import typing_extensions
from . import addons as addons
from . import extensions as extensions
from . import freedesktop as freedesktop
from . import grease_pencil as grease_pencil

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")



"""
This module contains callback lists


--------------------

This script shows the most simple example of adding a handler.

```../examples/bpy.app.handlers.py```


--------------------

By default handlers are freed when loading new files, in some cases you may
want the handler stay running across multiple files (when the handler is
part of an add-on for example).

For this the bpy.app.handlers.persistent decorator needs to be used.

```../examples/bpy.app.handlers.1.py```


--------------------

Altering data from handlers should be done carefully. While rendering the
frame_change_pre

 and frame_change_post

 handlers are called from one
thread and the viewport updates from a different thread. If the handler changes
data that is accessed by the viewport, this can cause a crash of Blender. In
such cases, lock the interface (Render → Lock Interface or
bpy.types.RenderSettings.use_lock_interface) before starting a render.

Below is an example of a mesh that is altered from a handler:

```../examples/bpy.app.handlers.2.py```

"""

import typing
import collections.abc
import typing_extensions
import bpy.types

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")
animation_playback_post: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on ending animation playback
"""

animation_playback_pre: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on starting animation playback
"""

annotation_post: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on drawing an annotation (after)
"""

annotation_pre: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on drawing an annotation (before)
"""

composite_cancel: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on a compositing background job (cancel)
"""

composite_post: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on a compositing background job (after)
"""

composite_pre: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on a compositing background job (before)
"""

depsgraph_update_post: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on depsgraph update (post)
"""

depsgraph_update_pre: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on depsgraph update (pre)
"""

frame_change_post: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" Called after frame change for playback and rendering, after the data has been evaluated for the new frame.
"""

frame_change_pre: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" Called after frame change for playback and rendering, before any data is evaluated for the new frame. This makes it possible to change data and relations (for example swap an object to another mesh) for the new frame. Note that this handler is not to be used as 'before the frame changes' event. The dependency graph is not available in this handler, as data and relations may have been altered and the dependency graph has not yet been updated for that.
"""

load_factory_preferences_post: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on loading factory preferences (after)
"""

load_factory_startup_post: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on loading factory startup (after)
"""

load_post: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on loading a new blend file (after). Accepts one argument: the file being loaded, an empty string for the startup-file.
"""

load_post_fail: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on failure to load a new blend file (after). Accepts one argument: the file being loaded, an empty string for the startup-file.
"""

load_pre: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on loading a new blend file (before).Accepts one argument: the file being loaded, an empty string for the startup-file.
"""

object_bake_cancel: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on canceling a bake job; will be called in the main thread
"""

object_bake_complete: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on completing a bake job; will be called in the main thread
"""

object_bake_pre: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" before starting a bake job
"""

persistent: typing.Any
""" Function decorator for callback functions not to be removed when loading new files
"""

redo_post: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on loading a redo step (after)
"""

redo_pre: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on loading a redo step (before)
"""

render_cancel: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on canceling a render job
"""

render_complete: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on completion of render job
"""

render_init: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on initialization of a render job
"""

render_post: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on render (after)
"""

render_pre: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on render (before)
"""

render_stats: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on printing render statistics. Accepts one argument: the render stats (render/saving time plus in background mode frame/used [peak] memory).
"""

render_write: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on writing a render frame (directly after the frame is written)
"""

save_post: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on saving a blend file (after). Accepts one argument: the file being saved, an empty string for the startup-file.
"""

save_post_fail: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on failure to save a blend file (after). Accepts one argument: the file being saved, an empty string for the startup-file.
"""

save_pre: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on saving a blend file (before). Accepts one argument: the file being saved, an empty string for the startup-file.
"""

translation_update_post: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on translation settings update
"""

undo_post: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on loading an undo step (after)
"""

undo_pre: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on loading an undo step (before)
"""

version_update: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on ending the versioning code
"""

xr_session_start_pre: list[collections.abc.Callable[[bpy.types.Scene], None]]
""" on starting an xr session (before)
"""



import typing
import collections.abc
import typing_extensions

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

def edges_from_elem(ele): ...
def elems_depth_measure(ele_dst, ele_src, other_edges_over_cb):
    """Returns·ele_dst vert depths from ele_src, aligned with ele_dst verts."""

    ...

def elems_depth_search(ele_init, depths, other_edges_over_cb, results_init=None):
    """List of depths -> List of elems that match those depths."""

    ...

def find_next(ele_dst, ele_src): ...
def other_edges_over_edge(e): ...
def other_edges_over_face(e): ...
def select_next(bm, report): ...
def select_prev(bm, report): ...
def verts_from_elem(ele): ...



"""
This module contains predicates operating on vertices (0D elements)
and polylines (1D elements).  It is also intended to be a collection
of examples for predicate definition in Python.

User-defined predicates inherit one of the following base classes,
depending on the object type (0D or 1D) to operate on and the arity
(unary or binary):

* freestyle.types.BinaryPredicate0D
* freestyle.types.BinaryPredicate1D
* freestyle.types.UnaryPredicate0D
* freestyle.types.UnaryPredicate1D

"""

import typing
import collections.abc
import typing_extensions
import freestyle.types

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

class AndBP1D: ...
class AndUP1D: ...

class ContourUP1D:
    """Class hierarchy: `freestyle.types.UnaryPredicate1D` > `ContourUP1D`"""

    def __call__(self, inter: freestyle.types.Interface1D) -> bool:
        """Returns true if the Interface1D is a contour. An Interface1D is a
        contour if it is bordered by a different shape on each of its sides.

                :param inter: An Interface1D object.
                :type inter: freestyle.types.Interface1D
                :return: True if the Interface1D is a contour, false otherwise.
                :rtype: bool
        """
        ...

class DensityLowerThanUP1D:
    """Class hierarchy: `freestyle.types.UnaryPredicate1D` > `DensityLowerThanUP1D`"""

    def __init__(self, threshold: float, sigma: float = 2.0):
        """Builds a DensityLowerThanUP1D object.

                :param threshold: The value of the threshold density. Any Interface1D
        having a density lower than this threshold will match.
                :type threshold: float
                :param sigma: The sigma value defining the density evaluation window
        size used in the `freestyle.functions.DensityF0D` functor.
                :type sigma: float
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> bool:
        """Returns true if the density evaluated for the Interface1D is less
        than a user-defined density value.

                :param inter: An Interface1D object.
                :type inter: freestyle.types.Interface1D
                :return: True if the density is lower than a threshold.
                :rtype: bool
        """
        ...

class EqualToChainingTimeStampUP1D:
    """Class hierarchy: `freestyle.types.UnaryPredicate1D` > `freestyle.types.EqualToChainingTimeStampUP1D`"""

    def __init__(self, ts: int):
        """Builds a EqualToChainingTimeStampUP1D object.

        :param ts: A time stamp value.
        :type ts: int
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> bool:
        """Returns true if the Interface1D's time stamp is equal to a certain
        user-defined value.

                :param inter: An Interface1D object.
                :type inter: freestyle.types.Interface1D
                :return: True if the time stamp is equal to a user-defined value.
                :rtype: bool
        """
        ...

class EqualToTimeStampUP1D:
    """Class hierarchy: `freestyle.types.UnaryPredicate1D` > `EqualToTimeStampUP1D`"""

    def __init__(self, ts: int):
        """Builds a EqualToTimeStampUP1D object.

        :param ts: A time stamp value.
        :type ts: int
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> bool:
        """Returns true if the Interface1D's time stamp is equal to a certain
        user-defined value.

                :param inter: An Interface1D object.
                :type inter: freestyle.types.Interface1D
                :return: True if the time stamp is equal to a user-defined value.
                :rtype: bool
        """
        ...

class ExternalContourUP1D:
    """Class hierarchy: `freestyle.types.UnaryPredicate1D` > `ExternalContourUP1D`"""

    def __call__(self, inter: freestyle.types.Interface1D) -> bool:
        """Returns true if the Interface1D is an external contour.
        An Interface1D is an external contour if it is bordered by no shape on
        one of its sides.

                :param inter: An Interface1D object.
                :type inter: freestyle.types.Interface1D
                :return: True if the Interface1D is an external contour, false
        otherwise.
                :rtype: bool
        """
        ...

class FalseBP1D:
    """Class hierarchy: `freestyle.types.BinaryPredicate1D` > `FalseBP1D`"""

    def __call__(
        self, inter1: freestyle.types.Interface1D, inter2: freestyle.types.Interface1D
    ) -> bool:
        """Always returns false.

        :param inter1: The first Interface1D object.
        :type inter1: freestyle.types.Interface1D
        :param inter2: The second Interface1D object.
        :type inter2: freestyle.types.Interface1D
        :return: False.
        :rtype: bool
        """
        ...

class FalseUP0D:
    """Class hierarchy: `freestyle.types.UnaryPredicate0D` > `FalseUP0D`"""

    def __call__(self, it: freestyle.types.Interface0DIterator) -> bool:
        """Always returns false.

        :param it: An Interface0DIterator object.
        :type it: freestyle.types.Interface0DIterator
        :return: False.
        :rtype: bool
        """
        ...

class FalseUP1D:
    """Class hierarchy: `freestyle.types.UnaryPredicate1D` > `FalseUP1D`"""

    def __call__(self, inter: freestyle.types.Interface1D) -> bool:
        """Always returns false.

        :param inter: An Interface1D object.
        :type inter: freestyle.types.Interface1D
        :return: False.
        :rtype: bool
        """
        ...

class Length2DBP1D:
    """Class hierarchy: `freestyle.types.BinaryPredicate1D` > `Length2DBP1D`"""

    def __call__(
        self, inter1: freestyle.types.Interface1D, inter2: freestyle.types.Interface1D
    ) -> bool:
        """Returns true if the 2D length of inter1 is less than the 2D length
        of inter2.

                :param inter1: The first Interface1D object.
                :type inter1: freestyle.types.Interface1D
                :param inter2: The second Interface1D object.
                :type inter2: freestyle.types.Interface1D
                :return: True or false.
                :rtype: bool
        """
        ...

class MaterialBP1D:
    """Checks whether the two supplied ViewEdges have the same material."""

    ...

class NotBP1D: ...
class NotUP1D: ...
class ObjectNamesUP1D: ...
class OrBP1D: ...
class OrUP1D: ...
class QuantitativeInvisibilityRangeUP1D: ...

class QuantitativeInvisibilityUP1D:
    """Class hierarchy: `freestyle.types.UnaryPredicate1D` > `QuantitativeInvisibilityUP1D`"""

    def __init__(self, qi: int = 0):
        """Builds a QuantitativeInvisibilityUP1D object.

                :param qi: The Quantitative Invisibility you want the Interface1D to
        have.
                :type qi: int
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> bool:
        """Returns true if the Quantitative Invisibility evaluated at an
        Interface1D, using the
        `freestyle.functions.QuantitativeInvisibilityF1D` functor,
        equals a certain user-defined value.

                :param inter: An Interface1D object.
                :type inter: freestyle.types.Interface1D
                :return: True if Quantitative Invisibility equals a user-defined
        value.
                :rtype: bool
        """
        ...

class SameShapeIdBP1D:
    """Class hierarchy: `freestyle.types.BinaryPredicate1D` > `SameShapeIdBP1D`"""

    def __call__(
        self, inter1: freestyle.types.Interface1D, inter2: freestyle.types.Interface1D
    ) -> bool:
        """Returns true if inter1 and inter2 belong to the same shape.

        :param inter1: The first Interface1D object.
        :type inter1: freestyle.types.Interface1D
        :param inter2: The second Interface1D object.
        :type inter2: freestyle.types.Interface1D
        :return: True or false.
        :rtype: bool
        """
        ...

class ShapeUP1D:
    """Class hierarchy: `freestyle.types.UnaryPredicate1D` > `ShapeUP1D`"""

    def __init__(self, first: int, second: int = 0):
        """Builds a ShapeUP1D object.

        :param first: The first Id component.
        :type first: int
        :param second: The second Id component.
        :type second: int
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> bool:
        """Returns true if the shape to which the Interface1D belongs to has the
        same `freestyle.types.Id` as the one specified by the user.

                :param inter: An Interface1D object.
                :type inter: freestyle.types.Interface1D
                :return: True if Interface1D belongs to the shape of the
        user-specified Id.
                :rtype: bool
        """
        ...

class TrueBP1D:
    """Class hierarchy: `freestyle.types.BinaryPredicate1D` > `TrueBP1D`"""

    def __call__(
        self, inter1: freestyle.types.Interface1D, inter2: freestyle.types.Interface1D
    ) -> bool:
        """Always returns true.

        :param inter1: The first Interface1D object.
        :type inter1: freestyle.types.Interface1D
        :param inter2: The second Interface1D object.
        :type inter2: freestyle.types.Interface1D
        :return: True.
        :rtype: bool
        """
        ...

class TrueUP0D:
    """Class hierarchy: `freestyle.types.UnaryPredicate0D` > `TrueUP0D`"""

    def __call__(self, it: freestyle.types.Interface0DIterator) -> bool:
        """Always returns true.

        :param it: An Interface0DIterator object.
        :type it: freestyle.types.Interface0DIterator
        :return: True.
        :rtype: bool
        """
        ...

class TrueUP1D:
    """Class hierarchy: `freestyle.types.UnaryPredicate1D` > `TrueUP1D`"""

    def __call__(self, inter: freestyle.types.Interface1D) -> bool:
        """Always returns true.

        :param inter: An Interface1D object.
        :type inter: freestyle.types.Interface1D
        :return: True.
        :rtype: bool
        """
        ...

class ViewMapGradientNormBP1D:
    """Class hierarchy: `freestyle.types.BinaryPredicate1D` > `ViewMapGradientNormBP1D`"""

    def __init__(
        self,
        level: int,
        integration_type: freestyle.types.IntegrationType = IntegrationType.MEAN,
        sampling: float = 2.0,
    ):
        """Builds a ViewMapGradientNormBP1D object.

                :param level: The level of the pyramid from which the pixel must be
        read.
                :type level: int
                :param integration_type: The integration method used to compute a single value
        from a set of values.
                :type integration_type: freestyle.types.IntegrationType
                :param sampling: The resolution used to sample the chain:
        GetViewMapGradientNormF0D is evaluated at each sample point and
        the result is obtained by combining the resulting values into a
        single one, following the method specified by integration_type.
                :type sampling: float
        """
        ...

    def __call__(
        self, inter1: freestyle.types.Interface1D, inter2: freestyle.types.Interface1D
    ) -> bool:
        """Returns true if the evaluation of the Gradient norm Function is
        higher for inter1 than for inter2.

                :param inter1: The first Interface1D object.
                :type inter1: freestyle.types.Interface1D
                :param inter2: The second Interface1D object.
                :type inter2: freestyle.types.Interface1D
                :return: True or false.
                :rtype: bool
        """
        ...

class WithinImageBoundaryUP1D:
    """Class hierarchy: `freestyle.types.UnaryPredicate1D` > `WithinImageBoundaryUP1D`"""

    def __init__(self, xmin: float, ymin: float, xmax: float, ymax: float):
        """Builds an WithinImageBoundaryUP1D object.

        :param xmin: X lower bound of the image boundary.
        :type xmin: float
        :param ymin: Y lower bound of the image boundary.
        :type ymin: float
        :param xmax: X upper bound of the image boundary.
        :type xmax: float
        :param ymax: Y upper bound of the image boundary.
        :type ymax: float
        """
        ...

    def __call__(self, inter):
        """Returns true if the Interface1D intersects with image boundary.

        :param inter:
        """
        ...

class pyBackTVertexUP0D:
    """Check whether an Interface0DIterator references a TVertex and is
    the one that is hidden (inferred from the context).
    """

    ...

class pyClosedCurveUP1D: ...
class pyDensityFunctorUP1D: ...
class pyDensityUP1D: ...
class pyDensityVariableSigmaUP1D: ...
class pyHighDensityAnisotropyUP1D: ...
class pyHighDirectionalViewMapDensityUP1D: ...
class pyHighSteerableViewMapDensityUP1D: ...
class pyHighViewMapDensityUP1D: ...
class pyHighViewMapGradientNormUP1D: ...
class pyHigherCurvature2DAngleUP0D: ...
class pyHigherLengthUP1D: ...
class pyHigherNumberOfTurnsUP1D: ...
class pyIsInOccludersListUP1D: ...
class pyIsOccludedByIdListUP1D: ...
class pyIsOccludedByItselfUP1D: ...
class pyIsOccludedByUP1D: ...
class pyLengthBP1D: ...
class pyLowDirectionalViewMapDensityUP1D: ...
class pyLowSteerableViewMapDensityUP1D: ...
class pyNFirstUP1D: ...
class pyNatureBP1D: ...
class pyNatureUP1D: ...
class pyParameterUP0D: ...
class pyParameterUP0DGoodOne: ...
class pyProjectedXBP1D: ...
class pyProjectedYBP1D: ...
class pyShapeIdListUP1D: ...
class pyShapeIdUP1D: ...
class pyShuffleBP1D: ...
class pySilhouetteFirstBP1D: ...
class pyUEqualsUP0D: ...
class pyVertexNatureUP0D: ...
class pyViewMapGradientNormBP1D: ...
class pyZBP1D: ...
class pyZDiscontinuityBP1D: ...
class pyZSmallerUP1D: ...



import typing
import collections.abc
import typing_extensions
import bpy.types

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

class PREFERENCES_OT_addon_disable(bpy.types.Operator):
    """Turn off this extension"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, _context):
        """

        :param _context:
        """
        ...

class PREFERENCES_OT_addon_enable(bpy.types.Operator):
    """Turn on this extension"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, _context):
        """

        :param _context:
        """
        ...

class PREFERENCES_OT_addon_expand(bpy.types.Operator):
    """Display information and preferences for this add-on"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, _context):
        """

        :param _context:
        """
        ...

class PREFERENCES_OT_addon_install(bpy.types.Operator):
    """Install an add-on"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

    def invoke(self, context, _event):
        """

        :param context:
        :param _event:
        """
        ...

class PREFERENCES_OT_addon_refresh(bpy.types.Operator):
    """Scan add-on directories for new modules"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, _context):
        """

        :param _context:
        """
        ...

class PREFERENCES_OT_addon_remove(bpy.types.Operator):
    """Delete the add-on from the file system"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, _context):
        """

        :param _context:
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

    def invoke(self, context, _event):
        """

        :param context:
        :param _event:
        """
        ...

    @staticmethod
    def path_from_addon(module):
        """

        :param module:
        """
        ...

class PREFERENCES_OT_addon_show(bpy.types.Operator):
    """Show add-on preferences"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

class PREFERENCES_OT_app_template_install(bpy.types.Operator):
    """Install an application template"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, _context):
        """

        :param _context:
        """
        ...

    def invoke(self, context, _event):
        """

        :param context:
        :param _event:
        """
        ...

class PREFERENCES_OT_copy_prev(bpy.types.Operator):
    """Copy settings from previous version"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, _context):
        """

        :param _context:
        """
        ...

    @classmethod
    def poll(cls, _context):
        """

        :param _context:
        """
        ...

    @classmethod
    def previous_version(cls): ...

class PREFERENCES_OT_keyconfig_activate(bpy.types.Operator):
    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, _context):
        """

        :param _context:
        """
        ...

class PREFERENCES_OT_keyconfig_export(bpy.types.Operator):
    """Export key configuration to a Python script"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

    def invoke(self, context, _event):
        """

        :param context:
        :param _event:
        """
        ...

class PREFERENCES_OT_keyconfig_import(bpy.types.Operator):
    """Import key configuration from a Python script"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, _context):
        """

        :param _context:
        """
        ...

    def invoke(self, context, _event):
        """

        :param context:
        :param _event:
        """
        ...

class PREFERENCES_OT_keyconfig_remove(bpy.types.Operator):
    """Remove key config"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """
        ...

class PREFERENCES_OT_keyconfig_test(bpy.types.Operator):
    """Test key configuration for conflicts"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

class PREFERENCES_OT_keyitem_add(bpy.types.Operator):
    """Add key map item"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

class PREFERENCES_OT_keyitem_remove(bpy.types.Operator):
    """Remove key map item"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """
        ...

class PREFERENCES_OT_keyitem_restore(bpy.types.Operator):
    """Restore key map item"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """
        ...

class PREFERENCES_OT_keymap_restore(bpy.types.Operator):
    """Restore key map(s)"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

class PREFERENCES_OT_script_directory_new(bpy.types.Operator):
    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

    def invoke(self, context, _event):
        """

        :param context:
        :param _event:
        """
        ...

class PREFERENCES_OT_script_directory_remove(bpy.types.Operator):
    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

class PREFERENCES_OT_studiolight_copy_settings(bpy.types.Operator):
    """Copy Studio Light settings to the Studio Light editor"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

class PREFERENCES_OT_studiolight_install(bpy.types.Operator):
    """Install a user defined light"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

    def invoke(self, context, _event):
        """

        :param context:
        :param _event:
        """
        ...

class PREFERENCES_OT_studiolight_new(bpy.types.Operator):
    """Save custom studio light from the studio light editor settings"""

    ask_override: typing.Any
    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, _context):
        """

        :param _context:
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

    def invoke(self, context, _event):
        """

        :param context:
        :param _event:
        """
        ...

class PREFERENCES_OT_studiolight_uninstall(bpy.types.Operator):
    """Delete Studio Light"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

class PREFERENCES_OT_theme_install(bpy.types.Operator):
    """Load and apply a Blender XML theme file"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, _context):
        """

        :param _context:
        """
        ...

    def invoke(self, context, _event):
        """

        :param context:
        :param _event:
        """
        ...



import typing
import collections.abc
import typing_extensions

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

def activate(*, template_id=None, reload_scripts=False): ...
def import_from_id(template_id, *, ignore_not_found=False): ...
def import_from_path(path, *, ignore_not_found=False): ...
def reset(*, reload_scripts=False):
    """Sets default state."""

    ...



import typing
import collections.abc
import typing_extensions
import bl_ui.properties_animviz
import bpy.types
import rna_prop_ui

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

class ARMATURE_MT_collection_context_menu(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, context):
        """

        :param context:
        """
        ...

class ARMATURE_MT_collection_tree_context_menu(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, context):
        """

        :param context:
        """
        ...

class ArmatureButtonsPanel:
    bl_context: typing.Any
    bl_region_type: typing.Any
    bl_space_type: typing.Any

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """
        ...

class DATA_PT_bone_collections(ArmatureButtonsPanel, bpy.types.Panel):
    bl_context: typing.Any
    bl_label: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, context):
        """

        :param context:
        """
        ...

class DATA_PT_context_arm(ArmatureButtonsPanel, bpy.types.Panel):
    bl_context: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, context):
        """

        :param context:
        """
        ...

class DATA_PT_custom_props_arm(
    rna_prop_ui.PropertyPanel, ArmatureButtonsPanel, bpy.types.Panel
):
    """The subclass should have its own poll function
    and the variable '_context_path' MUST be set.
    """

    COMPAT_ENGINES: typing.Any
    bl_context: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_order: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

class DATA_PT_custom_props_bcoll(
    rna_prop_ui.PropertyPanel, ArmatureButtonsPanel, bpy.types.Panel
):
    """The subclass should have its own poll function
    and the variable '_context_path' MUST be set.
    """

    COMPAT_ENGINES: typing.Any
    bl_context: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_order: typing.Any
    bl_parent_id: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """
        ...

class DATA_PT_display(ArmatureButtonsPanel, bpy.types.Panel):
    bl_context: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, context):
        """

        :param context:
        """
        ...

class DATA_PT_iksolver_itasc(ArmatureButtonsPanel, bpy.types.Panel):
    bl_context: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, context):
        """

        :param context:
        """
        ...

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """
        ...

class DATA_PT_motion_paths(
    bl_ui.properties_animviz.MotionPathButtonsPanel, bpy.types.Panel
):
    bl_context: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, context):
        """

        :param context:
        """
        ...

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """
        ...

class DATA_PT_motion_paths_display(
    bl_ui.properties_animviz.MotionPathButtonsPanel_display, bpy.types.Panel
):
    bl_context: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_parent_id: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, context):
        """

        :param context:
        """
        ...

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """
        ...

class DATA_PT_pose(ArmatureButtonsPanel, bpy.types.Panel):
    bl_context: typing.Any
    bl_label: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, context):
        """

        :param context:
        """
        ...

class DATA_UL_bone_collections(bpy.types.UIList):
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw_item(
        self,
        _context,
        layout,
        armature,
        bcoll,
        _icon,
        _active_data,
        _active_propname,
        _index,
    ):
        """

        :param _context:
        :param layout:
        :param armature:
        :param bcoll:
        :param _icon:
        :param _active_data:
        :param _active_propname:
        :param _index:
        """
        ...

class POSE_MT_selection_set_create(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, context):
        """

        :param context:
        """
        ...

class POSE_MT_selection_sets_context_menu(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, context):
        """

        :param context:
        """
        ...

class POSE_MT_selection_sets_select(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, context):
        """

        :param context:
        """
        ...

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """
        ...

class POSE_PT_selection_sets(bpy.types.Panel):
    bl_context: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, context):
        """

        :param context:
        """
        ...

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """
        ...

class POSE_UL_selection_set(bpy.types.UIList):
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw_item(
        self, context, layout, data, item, icon, active_data, active_propname, index
    ):
        """

        :param context:
        :param layout:
        :param data:
        :param item:
        :param icon:
        :param active_data:
        :param active_propname:
        :param index:
        """
        ...



import typing
import collections.abc
import typing_extensions
from . import bl_previews_render as bl_previews_render

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")



import typing
import collections.abc
import typing_extensions

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

def build_property_typemap(skip_classes, skip_typemap): ...
def print_ln(data): ...
def rna2xml(
    fw=None,
    root_node="",
    root_rna=None,
    root_rna_skip=set(),
    root_ident="",
    ident_val="  ",
    skip_classes=None,
    skip_typemap=None,
    pretty_format=True,
    method="DATA",
): ...
def xml2rna(root_xml, *, root_rna=None, secure_types=None): ...
def xml_file_run(context, filepath, rna_map, secure_types=None): ...
def xml_file_write(context, filepath, rna_map, *, skip_typemap=None): ...



import typing
import collections.abc
import typing_extensions
from . import progress_report as progress_report

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")



import typing
import collections.abc
import typing_extensions
import bpy.types

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

class SCENE_OT_freestyle_add_edge_marks_to_keying_set(bpy.types.Operator):
    """Add the data paths to the Freestyle Edge Mark property of selected edges to the active keying set"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """
        ...

class SCENE_OT_freestyle_add_face_marks_to_keying_set(bpy.types.Operator):
    """Add the data paths to the Freestyle Face Mark property of selected polygons to the active keying set"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """
        ...

class SCENE_OT_freestyle_fill_range_by_selection(bpy.types.Operator):
    """Fill the Range Min/Max entries by the min/max distance between selected mesh objects and the source object (either a user-specified object or the active camera)"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """
        ...

class SCENE_OT_freestyle_module_open(bpy.types.Operator):
    """Open a style module file"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, _context):
        """

        :param _context:
        """
        ...

    def invoke(self, context, _event):
        """

        :param context:
        :param _event:
        """
        ...

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """
        ...



import typing
import collections.abc
import typing_extensions
import bpy.types

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

class SPREADSHEET_HT_header(bpy.types.Header):
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, context):
        """

        :param context:
        """
        ...

    def draw_collapsed_viewer_path(self, context, layout, viewer_path):
        """

        :param context:
        :param layout:
        :param viewer_path:
        """
        ...

    def draw_full_viewer_path(self, context, layout, viewer_path):
        """

        :param context:
        :param layout:
        :param viewer_path:
        """
        ...

    def draw_spreadsheet_context(self, layout, ctx):
        """

        :param layout:
        :param ctx:
        """
        ...

    def draw_spreadsheet_viewer_path_icon(self, layout, space, icon="RIGHTARROW_THIN"):
        """

        :param layout:
        :param space:
        :param icon:
        """
        ...

    def draw_without_viewer_path(self, layout):
        """

        :param layout:
        """
        ...

    def selection_filter_available(self, space):
        """

        :param space:
        """
        ...



"""

--------------------


--------------------


--------------------


--------------------


--------------------


--------------------

"""

import typing
import collections.abc
import typing_extensions
import bpy.types
import mathutils

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

class BMDeformVert:
    def clear(self):
        """Clears all weights."""
        ...

    def get(self, key: int, default=None):
        """Returns the deform weight matching the key or default
        when not found (matches Python's dictionary function of the same name).

                :param key: The key associated with deform weight.
                :type key: int
                :param default: Optional argument for the value to return if
        key is not found.
        """
        ...

    def items(self):
        """Return (group, weight) pairs for this vertex
        (matching Python's dict.items() functionality).

                :return: (key, value) pairs for each deform weight of this vertex.
        """
        ...

    def keys(self) -> list[int]:
        """Return the group indices used by this vertex
        (matching Python's dict.keys() functionality).

                :return: the deform group this vertex uses
                :rtype: list[int]
        """
        ...

    def values(self) -> list[float]:
        """Return the weights of the deform vertex
        (matching Python's dict.values() functionality).

                :return: The weights that influence this vertex
                :rtype: list[float]
        """
        ...

class BMEdge:
    """The BMesh edge connecting 2 verts"""

    hide: bool
    """ Hidden state of this element.

    :type: bool
    """

    index: int
    """ Index of this element.

    :type: int
    """

    is_boundary: bool
    """ True when this edge is at the boundary of a face (read-only).

    :type: bool
    """

    is_contiguous: bool
    """ True when this edge is manifold, between two faces with the same winding (read-only).

    :type: bool
    """

    is_convex: bool
    """ True when this edge joins two convex faces, depends on a valid face normal (read-only).

    :type: bool
    """

    is_manifold: bool
    """ True when this edge is manifold (read-only).

    :type: bool
    """

    is_valid: bool
    """ True when this element is valid (hasn't been removed).

    :type: bool
    """

    is_wire: bool
    """ True when this edge is not connected to any faces (read-only).

    :type: bool
    """

    link_faces: BMElemSeq[BMFace]
    """ Faces connected to this edge, (read-only).

    :type: BMElemSeq[BMFace]
    """

    link_loops: BMElemSeq[BMLoop]
    """ Loops connected to this edge, (read-only).

    :type: BMElemSeq[BMLoop]
    """

    seam: bool
    """ Seam for UV unwrapping.

    :type: bool
    """

    select: bool
    """ Selected state of this element.

    :type: bool
    """

    smooth: bool
    """ Smooth state of this element.

    :type: bool
    """

    tag: bool
    """ Generic attribute scripts can use for own logic

    :type: bool
    """

    verts: BMElemSeq[BMVert]
    """ Verts this edge uses (always 2), (read-only).

    :type: BMElemSeq[BMVert]
    """

    def calc_face_angle(self, fallback: typing.Any = None) -> float:
        """

                :param fallback: return this when the edge doesn't have 2 faces
        (instead of raising a `ValueError`).
                :type fallback: typing.Any
                :return: The angle between 2 connected faces in radians.
                :rtype: float
        """
        ...

    def calc_face_angle_signed(self, fallback: typing.Any = None) -> float:
        """

                :param fallback: return this when the edge doesn't have 2 faces
        (instead of raising a `ValueError`).
                :type fallback: typing.Any
                :return: The angle between 2 connected faces in radians (negative for concave join).
                :rtype: float
        """
        ...

    def calc_length(self) -> float:
        """

        :return: The length between both verts.
        :rtype: float
        """
        ...

    def calc_tangent(self, loop: BMLoop) -> mathutils.Vector:
        """Return the tangent at this edge relative to a face (pointing inward into the face).
        This uses the face normal for calculation.

                :param loop: The loop used for tangent calculation.
                :type loop: BMLoop
                :return: a normalized vector.
                :rtype: mathutils.Vector
        """
        ...

    def copy_from(self, other: typing_extensions.Self):
        """Copy values from another element of matching type.

        :param other:
        :type other: typing_extensions.Self
        """
        ...

    def hide_set(self, hide: bool):
        """Set the hide state.
        This is different from the hide attribute because it updates the selection and hide state of associated geometry.

                :param hide: Hidden or visible.
                :type hide: bool
        """
        ...

    def normal_update(self):
        """Update normals of all connected faces and the edge verts."""
        ...

    def other_vert(self, vert: BMVert) -> BMVert:
        """Return the other vertex on this edge or None if the vertex is not used by this edge.

        :param vert: a vert in this edge.
        :type vert: BMVert
        :return: The edges other vert.
        :rtype: BMVert
        """
        ...

    def select_set(self, select: bool):
        """Set the selection.
        This is different from the select attribute because it updates the selection state of associated geometry.

                :param select: Select or de-select.
                :type select: bool
        """
        ...

    def __getitem__(self, key: BMLayerItem) -> typing.Any:
        """

        :param key:
        :type key: BMLayerItem
        :return:
        :rtype: typing.Any
        """
        ...

    def __setitem__(self, key: BMLayerItem, value: typing.Any):
        """

        :param key:
        :type key: BMLayerItem
        :param value:
        :type value: typing.Any
        """
        ...

    def __delitem__(self, key: BMLayerItem):
        """

        :param key:
        :type key: BMLayerItem
        """
        ...

class BMEdgeSeq:
    layers: BMLayerAccessEdge
    """ custom-data layers (read-only).

    :type: BMLayerAccessEdge
    """

    def ensure_lookup_table(self):
        """Ensure internal data needed for int subscription is initialized with verts/edges/faces, eg bm.verts[index].This needs to be called again after adding/removing data in this sequence."""
        ...

    def get(self, verts: list[BMVert], fallback=None) -> BMEdge:
        """Return an edge which uses the verts passed.

        :param verts: Sequence of verts.
        :type verts: list[BMVert]
        :param fallback: Return this value if nothing is found.
        :return: The edge found or None
        :rtype: BMEdge
        """
        ...

    def index_update(self):
        """Initialize the index values of this sequence.This is the equivalent of looping over all elements and assigning the index values."""
        ...

    def new(
        self, verts: tuple[BMVert, BMVert], example: BMEdge | None = None
    ) -> BMEdge:
        """Create a new edge from a given pair of verts.

        :param verts: Vertex pair.
        :type verts: tuple[BMVert, BMVert]
        :param example: Existing edge to initialize settings (optional argument).
        :type example: BMEdge | None
        :return: The newly created edge.
        :rtype: BMEdge
        """
        ...

    def remove(self, edge: BMEdge):
        """Remove an edge.

        :param edge:
        :type edge: BMEdge
        """
        ...

    def sort(self, key=None, reverse: bool = False):
        """Sort the elements of this sequence, using an optional custom sort key.
        Indices of elements are not changed, `BMElemSeq.index_update` can be used for that.

                :param key: The key that sets the ordering of the elements.
                :param reverse: Reverse the order of the elements
                :type reverse: bool
        """
        ...

    @typing.overload
    def __getitem__(self, key: int) -> BMEdge:
        """

        :param key:
        :type key: int
        :return:
        :rtype: BMEdge
        """
        ...

    @typing.overload
    def __getitem__(self, key: slice) -> tuple[BMEdge, ...]:
        """

        :param key:
        :type key: slice
        :return:
        :rtype: tuple[BMEdge, ...]
        """
        ...

    def __iter__(self) -> BMIter[BMEdge]:
        """

        :return:
        :rtype: BMIter[BMEdge]
        """
        ...

    def __len__(self) -> int:
        """

        :return:
        :rtype: int
        """
        ...

class BMEditSelIter: ...

class BMEditSelSeq:
    active: BMEdge | BMFace | BMVert
    """ The last selected element or None (read-only).

    :type: BMEdge | BMFace | BMVert
    """

    def add(self, element):
        """Add an element to the selection history (no action taken if its already added).

        :param element:
        """
        ...

    def clear(self):
        """Empties the selection history."""
        ...

    def discard(self, element):
        """Discard an element from the selection history.Like remove but doesn't raise an error when the elements not in the selection list.

        :param element:
        """
        ...

    def remove(self, element):
        """Remove an element from the selection history.

        :param element:
        """
        ...

    def validate(self):
        """Ensures all elements in the selection history are selected."""
        ...

class BMElemSeq(typing.Generic[GenericType1]):
    """General sequence type used for accessing any sequence of
    `BMVert`, `BMEdge`, `BMFace`, `BMLoop`.When accessed via `BMesh.verts`, `BMesh.edges`, `BMesh.faces`
    there are also functions to create/remove items.
    """

    def index_update(self):
        """Initialize the index values of this sequence.This is the equivalent of looping over all elements and assigning the index values."""
        ...

    @typing.overload
    def __getitem__(self, key: int) -> GenericType1:
        """

        :param key:
        :type key: int
        :return:
        :rtype: GenericType1
        """
        ...

    @typing.overload
    def __getitem__(self, key: slice) -> tuple[GenericType1, ...]:
        """

        :param key:
        :type key: slice
        :return:
        :rtype: tuple[GenericType1, ...]
        """
        ...

    def __iter__(self) -> BMIter[GenericType1]:
        """

        :return:
        :rtype: BMIter[GenericType1]
        """
        ...

    def __len__(self) -> int:
        """

        :return:
        :rtype: int
        """
        ...

class BMFace:
    """The BMesh face with 3 or more sides"""

    edges: BMElemSeq[BMEdge]
    """ Edges of this face, (read-only).

    :type: BMElemSeq[BMEdge]
    """

    hide: bool
    """ Hidden state of this element.

    :type: bool
    """

    index: int
    """ Index of this element.

    :type: int
    """

    is_valid: bool
    """ True when this element is valid (hasn't been removed).

    :type: bool
    """

    loops: BMElemSeq[BMLoop]
    """ Loops of this face, (read-only).

    :type: BMElemSeq[BMLoop]
    """

    material_index: int
    """ The face's material index.

    :type: int
    """

    normal: mathutils.Vector
    """ The normal for this face as a 3D, wrapped vector.

    :type: mathutils.Vector
    """

    select: bool
    """ Selected state of this element.

    :type: bool
    """

    smooth: bool
    """ Smooth state of this element.

    :type: bool
    """

    tag: bool
    """ Generic attribute scripts can use for own logic

    :type: bool
    """

    verts: BMElemSeq[BMVert]
    """ Verts of this face, (read-only).

    :type: BMElemSeq[BMVert]
    """

    def calc_area(self) -> float:
        """Return the area of the face.

        :return: Return the area of the face.
        :rtype: float
        """
        ...

    def calc_center_bounds(self) -> mathutils.Vector:
        """Return bounds center of the face.

        :return: a 3D vector.
        :rtype: mathutils.Vector
        """
        ...

    def calc_center_median(self) -> mathutils.Vector:
        """Return median center of the face.

        :return: a 3D vector.
        :rtype: mathutils.Vector
        """
        ...

    def calc_center_median_weighted(self) -> mathutils.Vector:
        """Return median center of the face weighted by edge lengths.

        :return: a 3D vector.
        :rtype: mathutils.Vector
        """
        ...

    def calc_perimeter(self) -> float:
        """Return the perimeter of the face.

        :return: Return the perimeter of the face.
        :rtype: float
        """
        ...

    def calc_tangent_edge(self) -> mathutils.Vector:
        """Return face tangent based on longest edge.

        :return: a normalized vector.
        :rtype: mathutils.Vector
        """
        ...

    def calc_tangent_edge_diagonal(self) -> mathutils.Vector:
        """Return face tangent based on the edge farthest from any vertex.

        :return: a normalized vector.
        :rtype: mathutils.Vector
        """
        ...

    def calc_tangent_edge_pair(self) -> mathutils.Vector:
        """Return face tangent based on the two longest disconnected edges.

        :return: a normalized vector.
        :rtype: mathutils.Vector
        """
        ...

    def calc_tangent_vert_diagonal(self) -> mathutils.Vector:
        """Return face tangent based on the two most distant vertices.

        :return: a normalized vector.
        :rtype: mathutils.Vector
        """
        ...

    def copy(self, verts: bool = True, edges: bool = True) -> typing_extensions.Self:
        """Make a copy of this face.

        :param verts: When set, the faces verts will be duplicated too.
        :type verts: bool
        :param edges: When set, the faces edges will be duplicated too.
        :type edges: bool
        :return: The newly created face.
        :rtype: typing_extensions.Self
        """
        ...

    def copy_from(self, other: typing_extensions.Self):
        """Copy values from another element of matching type.

        :param other:
        :type other: typing_extensions.Self
        """
        ...

    def copy_from_face_interp(self, face: typing_extensions.Self, vert: bool = True):
        """Interpolate the customdata from another face onto this one (faces should overlap).

        :param face: The face to interpolate data from.
        :type face: typing_extensions.Self
        :param vert: When True, also copy vertex data.
        :type vert: bool
        """
        ...

    def hide_set(self, hide: bool):
        """Set the hide state.
        This is different from the hide attribute because it updates the selection and hide state of associated geometry.

                :param hide: Hidden or visible.
                :type hide: bool
        """
        ...

    def normal_flip(self):
        """Reverses winding of a face, which flips its normal."""
        ...

    def normal_update(self):
        """Update face normal based on the positions of the face verts.
        This does not update the normals of face verts.

        """
        ...

    def select_set(self, select: bool):
        """Set the selection.
        This is different from the select attribute because it updates the selection state of associated geometry.

                :param select: Select or de-select.
                :type select: bool
        """
        ...

    def __getitem__(self, key: BMLayerItem) -> typing.Any:
        """

        :param key:
        :type key: BMLayerItem
        :return:
        :rtype: typing.Any
        """
        ...

    def __setitem__(self, key: BMLayerItem, value: typing.Any):
        """

        :param key:
        :type key: BMLayerItem
        :param value:
        :type value: typing.Any
        """
        ...

    def __delitem__(self, key: BMLayerItem):
        """

        :param key:
        :type key: BMLayerItem
        """
        ...

class BMFaceSeq:
    active: BMFace | None
    """ active face.

    :type: BMFace | None
    """

    layers: BMLayerAccessFace
    """ custom-data layers (read-only).

    :type: BMLayerAccessFace
    """

    def ensure_lookup_table(self):
        """Ensure internal data needed for int subscription is initialized with verts/edges/faces, eg bm.verts[index].This needs to be called again after adding/removing data in this sequence."""
        ...

    def get(self, verts: list[BMVert], fallback=None) -> BMFace:
        """Return a face which uses the verts passed.

        :param verts: Sequence of verts.
        :type verts: list[BMVert]
        :param fallback: Return this value if nothing is found.
        :return: The face found or None
        :rtype: BMFace
        """
        ...

    def index_update(self):
        """Initialize the index values of this sequence.This is the equivalent of looping over all elements and assigning the index values."""
        ...

    def new(self, verts: list[BMVert], example: BMFace | None = None) -> BMFace:
        """Create a new face from a given set of verts.

        :param verts: Sequence of 3 or more verts.
        :type verts: list[BMVert]
        :param example: Existing face to initialize settings (optional argument).
        :type example: BMFace | None
        :return: The newly created face.
        :rtype: BMFace
        """
        ...

    def remove(self, face: BMFace):
        """Remove a face.

        :param face:
        :type face: BMFace
        """
        ...

    def sort(self, key=None, reverse: bool = False):
        """Sort the elements of this sequence, using an optional custom sort key.
        Indices of elements are not changed, `BMElemSeq.index_update` can be used for that.

                :param key: The key that sets the ordering of the elements.
                :param reverse: Reverse the order of the elements
                :type reverse: bool
        """
        ...

    @typing.overload
    def __getitem__(self, key: int) -> BMFace:
        """

        :param key:
        :type key: int
        :return:
        :rtype: BMFace
        """
        ...

    @typing.overload
    def __getitem__(self, key: slice) -> tuple[BMFace, ...]:
        """

        :param key:
        :type key: slice
        :return:
        :rtype: tuple[BMFace, ...]
        """
        ...

    def __iter__(self) -> BMIter[BMFace]:
        """

        :return:
        :rtype: BMIter[BMFace]
        """
        ...

    def __len__(self) -> int:
        """

        :return:
        :rtype: int
        """
        ...

class BMIter(typing.Generic[GenericType1]):
    """Internal BMesh type for looping over verts/faces/edges,
    used for iterating over `BMElemSeq` types.
    """

    def __iter__(self) -> BMIter[GenericType1]:
        """

        :return:
        :rtype: BMIter[GenericType1]
        """
        ...

    def __next__(self) -> GenericType1:
        """

        :return:
        :rtype: GenericType1
        """
        ...

class BMLayerAccessEdge:
    """Exposes custom-data layer attributes."""

    color: BMLayerCollection
    """ Generic RGBA color with 8-bit precision custom-data layer.

    :type: BMLayerCollection
    """

    float: BMLayerCollection
    """ Generic float custom-data layer.

    :type: BMLayerCollection
    """

    float_color: BMLayerCollection
    """ Generic RGBA color with float precision custom-data layer.

    :type: BMLayerCollection
    """

    float_vector: BMLayerCollection
    """ Generic 3D vector with float precision custom-data layer.

    :type: BMLayerCollection
    """

    freestyle: BMLayerCollection
    """ Accessor for Freestyle edge layer.

    :type: BMLayerCollection
    """

    int: BMLayerCollection
    """ Generic int custom-data layer.

    :type: BMLayerCollection
    """

    string: BMLayerCollection
    """ Generic string custom-data layer (exposed as bytes, 255 max length).

    :type: BMLayerCollection
    """

class BMLayerAccessFace:
    """Exposes custom-data layer attributes."""

    color: BMLayerCollection
    """ Generic RGBA color with 8-bit precision custom-data layer.

    :type: BMLayerCollection
    """

    float: BMLayerCollection
    """ Generic float custom-data layer.

    :type: BMLayerCollection
    """

    float_color: BMLayerCollection
    """ Generic RGBA color with float precision custom-data layer.

    :type: BMLayerCollection
    """

    float_vector: BMLayerCollection
    """ Generic 3D vector with float precision custom-data layer.

    :type: BMLayerCollection
    """

    freestyle: BMLayerCollection
    """ Accessor for Freestyle face layer.

    :type: BMLayerCollection
    """

    int: BMLayerCollection
    """ Generic int custom-data layer.

    :type: BMLayerCollection
    """

    string: BMLayerCollection
    """ Generic string custom-data layer (exposed as bytes, 255 max length).

    :type: BMLayerCollection
    """

class BMLayerAccessLoop:
    """Exposes custom-data layer attributes."""

    color: BMLayerCollection
    """ Generic RGBA color with 8-bit precision custom-data layer.

    :type: BMLayerCollection
    """

    float: BMLayerCollection
    """ Generic float custom-data layer.

    :type: BMLayerCollection
    """

    float_color: BMLayerCollection
    """ Generic RGBA color with float precision custom-data layer.

    :type: BMLayerCollection
    """

    float_vector: BMLayerCollection
    """ Generic 3D vector with float precision custom-data layer.

    :type: BMLayerCollection
    """

    int: BMLayerCollection
    """ Generic int custom-data layer.

    :type: BMLayerCollection
    """

    string: BMLayerCollection
    """ Generic string custom-data layer (exposed as bytes, 255 max length).

    :type: BMLayerCollection
    """

    uv: BMLayerCollection
    """ Accessor for `BMLoopUV` UV (as a 2D Vector).

    :type: BMLayerCollection
    """

class BMLayerAccessVert:
    """Exposes custom-data layer attributes."""

    color: BMLayerCollection
    """ Generic RGBA color with 8-bit precision custom-data layer.

    :type: BMLayerCollection
    """

    deform: BMLayerCollection
    """ Vertex deform weight `BMDeformVert` (TODO).

    :type: BMLayerCollection
    """

    float: BMLayerCollection
    """ Generic float custom-data layer.

    :type: BMLayerCollection
    """

    float_color: BMLayerCollection
    """ Generic RGBA color with float precision custom-data layer.

    :type: BMLayerCollection
    """

    float_vector: BMLayerCollection
    """ Generic 3D vector with float precision custom-data layer.

    :type: BMLayerCollection
    """

    int: BMLayerCollection
    """ Generic int custom-data layer.

    :type: BMLayerCollection
    """

    shape: BMLayerCollection
    """ Vertex shapekey absolute location (as a 3D Vector).

    :type: BMLayerCollection
    """

    skin: BMLayerCollection
    """ Accessor for skin layer.

    :type: BMLayerCollection
    """

    string: BMLayerCollection
    """ Generic string custom-data layer (exposed as bytes, 255 max length).

    :type: BMLayerCollection
    """

class BMLayerCollection:
    """Gives access to a collection of custom-data layers of the same type and behaves like Python dictionaries, except for the ability to do list like index access."""

    active: BMLayerItem
    """ The active layer of this type (read-only).

    :type: BMLayerItem
    """

    is_singleton: bool
    """ True if there can exists only one layer of this type (read-only).

    :type: bool
    """

    def get(self, key: str, default: GenericType2 = None) -> BMLayerItem | GenericType2:
        """Returns the value of the layer matching the key or default
        when not found (matches Python's dictionary function of the same name).

                :param key: The key associated with the layer.
                :type key: str
                :param default: Optional argument for the value to return if
        key is not found.
                :type default: GenericType2
                :return:
                :rtype: BMLayerItem | GenericType2
        """
        ...

    def items(self):
        """Return the identifiers of collection members
        (matching Python's dict.items() functionality).

                :return: (key, value) pairs for each member of this collection.
        """
        ...

    def keys(self) -> list[str]:
        """Return the identifiers of collection members
        (matching Python's dict.keys() functionality).

                :return: the identifiers for each member of this collection.
                :rtype: list[str]
        """
        ...

    def new(self, name: str | None = "") -> BMLayerItem:
        """Create a new layer

        :param name: Optional name argument (will be made unique).
        :type name: str | None
        :return: The newly created layer.
        :rtype: BMLayerItem
        """
        ...

    def remove(self, layer: BMLayerItem):
        """Remove a layer

        :param layer: The layer to remove.
        :type layer: BMLayerItem
        """
        ...

    def values(self) -> list:
        """Return the values of collection
        (matching Python's dict.values() functionality).

                :return: the members of this collection.
                :rtype: list
        """
        ...

    def verify(self) -> BMLayerItem:
        """Create a new layer or return an existing active layer

        :return: The newly verified layer.
        :rtype: BMLayerItem
        """
        ...

class BMLayerItem:
    """Exposes a single custom data layer, their main purpose is for use as item accessors to custom-data when used with vert/edge/face/loop data."""

    name: str
    """ The layers unique name (read-only).

    :type: str
    """

    def copy_from(self, other: typing_extensions.Self):
        """Return a copy of the layer

        :param other: Another layer to copy from.
        :type other: typing_extensions.Self
        """
        ...

class BMLoop:
    """This is normally accessed from `BMFace.loops` where each face loop represents a corner of the face."""

    edge: BMEdge
    """ The loop's edge (between this loop and the next), (read-only).

    :type: BMEdge
    """

    face: BMFace
    """ The face this loop makes (read-only).

    :type: BMFace
    """

    index: int
    """ Index of this element.

    :type: int
    """

    is_convex: bool
    """ True when this loop is at the convex corner of a face, depends on a valid face normal (read-only).

    :type: bool
    """

    is_valid: bool
    """ True when this element is valid (hasn't been removed).

    :type: bool
    """

    link_loop_next: typing_extensions.Self
    """ The next face corner (read-only).

    :type: typing_extensions.Self
    """

    link_loop_prev: typing_extensions.Self
    """ The previous face corner (read-only).

    :type: typing_extensions.Self
    """

    link_loop_radial_next: typing_extensions.Self
    """ The next loop around the edge (read-only).

    :type: typing_extensions.Self
    """

    link_loop_radial_prev: typing_extensions.Self
    """ The previous loop around the edge (read-only).

    :type: typing_extensions.Self
    """

    link_loops: BMElemSeq[BMLoop]
    """ Loops connected to this loop, (read-only).

    :type: BMElemSeq[BMLoop]
    """

    tag: bool
    """ Generic attribute scripts can use for own logic

    :type: bool
    """

    vert: BMVert
    """ The loop's vertex (read-only).

    :type: BMVert
    """

    def calc_angle(self) -> float:
        """Return the angle at this loops corner of the face.
        This is calculated so sharper corners give lower angles.

                :return: The angle in radians.
                :rtype: float
        """
        ...

    def calc_normal(self) -> mathutils.Vector:
        """Return normal at this loops corner of the face.
        Falls back to the face normal for straight lines.

                :return: a normalized vector.
                :rtype: mathutils.Vector
        """
        ...

    def calc_tangent(self) -> mathutils.Vector:
        """Return the tangent at this loops corner of the face (pointing inward into the face).
        Falls back to the face normal for straight lines.

                :return: a normalized vector.
                :rtype: mathutils.Vector
        """
        ...

    def copy_from(self, other: typing_extensions.Self):
        """Copy values from another element of matching type.

        :param other:
        :type other: typing_extensions.Self
        """
        ...

    def copy_from_face_interp(
        self, face: BMFace, vert: bool = True, multires: bool = True
    ):
        """Interpolate the customdata from a face onto this loop (the loops vert should overlap the face).

        :param face: The face to interpolate data from.
        :type face: BMFace
        :param vert: When enabled, interpolate the loops vertex data (optional).
        :type vert: bool
        :param multires: When enabled, interpolate the loops multires data (optional).
        :type multires: bool
        """
        ...

    def __getitem__(self, key: BMLayerItem) -> typing.Any:
        """

        :param key:
        :type key: BMLayerItem
        :return:
        :rtype: typing.Any
        """
        ...

    def __setitem__(self, key: BMLayerItem, value: typing.Any):
        """

        :param key:
        :type key: BMLayerItem
        :param value:
        :type value: typing.Any
        """
        ...

    def __delitem__(self, key: BMLayerItem):
        """

        :param key:
        :type key: BMLayerItem
        """
        ...

class BMLoopSeq:
    layers: BMLayerAccessLoop
    """ custom-data layers (read-only).

    :type: BMLayerAccessLoop
    """

class BMLoopUV:
    pin_uv: bool
    """ UV pin state.

    :type: bool
    """

    select: bool
    """ UV select state.

    :type: bool
    """

    select_edge: bool
    """ UV edge select state.

    :type: bool
    """

    uv: mathutils.Vector
    """ Loops UV (as a 2D Vector).

    :type: mathutils.Vector
    """

class BMVert:
    """The BMesh vertex type"""

    co: mathutils.Vector
    """ The coordinates for this vertex as a 3D, wrapped vector.

    :type: mathutils.Vector
    """

    hide: bool
    """ Hidden state of this element.

    :type: bool
    """

    index: int
    """ Index of this element.

    :type: int
    """

    is_boundary: bool
    """ True when this vertex is connected to boundary edges (read-only).

    :type: bool
    """

    is_manifold: bool
    """ True when this vertex is manifold (read-only).

    :type: bool
    """

    is_valid: bool
    """ True when this element is valid (hasn't been removed).

    :type: bool
    """

    is_wire: bool
    """ True when this vertex is not connected to any faces (read-only).

    :type: bool
    """

    link_edges: BMElemSeq[BMEdge]
    """ Edges connected to this vertex (read-only).

    :type: BMElemSeq[BMEdge]
    """

    link_faces: BMElemSeq[BMFace]
    """ Faces connected to this vertex (read-only).

    :type: BMElemSeq[BMFace]
    """

    link_loops: BMElemSeq[BMLoop]
    """ Loops that use this vertex (read-only).

    :type: BMElemSeq[BMLoop]
    """

    normal: mathutils.Vector
    """ The normal for this vertex as a 3D, wrapped vector.

    :type: mathutils.Vector
    """

    select: bool
    """ Selected state of this element.

    :type: bool
    """

    tag: bool
    """ Generic attribute scripts can use for own logic

    :type: bool
    """

    def calc_edge_angle(self, fallback: typing.Any = None) -> float:
        """Return the angle between this vert's two connected edges.

                :param fallback: return this when the vert doesn't have 2 edges
        (instead of raising a `ValueError`).
                :type fallback: typing.Any
                :return: Angle between edges in radians.
                :rtype: float
        """
        ...

    def calc_shell_factor(self) -> float:
        """Return a multiplier calculated based on the sharpness of the vertex.
        Where a flat surface gives 1.0, and higher values sharper edges.
        This is used to maintain shell thickness when offsetting verts along their normals.

                :return: offset multiplier
                :rtype: float
        """
        ...

    def copy_from(self, other: typing_extensions.Self):
        """Copy values from another element of matching type.

        :param other:
        :type other: typing_extensions.Self
        """
        ...

    def copy_from_face_interp(self, face: BMFace):
        """Interpolate the customdata from a face onto this loop (the loops vert should overlap the face).

        :param face: The face to interpolate data from.
        :type face: BMFace
        """
        ...

    def copy_from_vert_interp(self, vert_pair: tuple[BMVert, BMVert], fac: float):
        """Interpolate the customdata from a vert between 2 other verts.

        :param vert_pair: The verts between which to interpolate data from.
        :type vert_pair: tuple[BMVert, BMVert]
        :param fac:
        :type fac: float
        """
        ...

    def hide_set(self, hide: bool):
        """Set the hide state.
        This is different from the hide attribute because it updates the selection and hide state of associated geometry.

                :param hide: Hidden or visible.
                :type hide: bool
        """
        ...

    def normal_update(self):
        """Update vertex normal.
        This does not update the normals of adjoining faces.

        """
        ...

    def select_set(self, select: bool):
        """Set the selection.
        This is different from the select attribute because it updates the selection state of associated geometry.

                :param select: Select or de-select.
                :type select: bool
        """
        ...

    def __getitem__(self, key: BMLayerItem) -> typing.Any:
        """

        :param key:
        :type key: BMLayerItem
        :return:
        :rtype: typing.Any
        """
        ...

    def __setitem__(self, key: BMLayerItem, value: typing.Any):
        """

        :param key:
        :type key: BMLayerItem
        :param value:
        :type value: typing.Any
        """
        ...

    def __delitem__(self, key: BMLayerItem):
        """

        :param key:
        :type key: BMLayerItem
        """
        ...

class BMVertSeq:
    layers: BMLayerAccessVert
    """ custom-data layers (read-only).

    :type: BMLayerAccessVert
    """

    def ensure_lookup_table(self):
        """Ensure internal data needed for int subscription is initialized with verts/edges/faces, eg bm.verts[index].This needs to be called again after adding/removing data in this sequence."""
        ...

    def index_update(self):
        """Initialize the index values of this sequence.This is the equivalent of looping over all elements and assigning the index values."""
        ...

    def new(
        self,
        co: collections.abc.Sequence[float] | mathutils.Vector = (0.0, 0.0, 0.0),
        example: BMVert = None,
    ) -> BMVert:
        """Create a new vertex.

        :param co: The initial location of the vertex (optional argument).
        :type co: collections.abc.Sequence[float] | mathutils.Vector
        :param example: Existing vert to initialize settings.
        :type example: BMVert
        :return: The newly created vertex.
        :rtype: BMVert
        """
        ...

    def remove(self, vert: BMVert):
        """Remove a vert.

        :param vert:
        :type vert: BMVert
        """
        ...

    def sort(self, key=None, reverse: bool = False):
        """Sort the elements of this sequence, using an optional custom sort key.
        Indices of elements are not changed, `BMElemSeq.index_update` can be used for that.

                :param key: The key that sets the ordering of the elements.
                :param reverse: Reverse the order of the elements
                :type reverse: bool
        """
        ...

    @typing.overload
    def __getitem__(self, key: int) -> BMVert:
        """

        :param key:
        :type key: int
        :return:
        :rtype: BMVert
        """
        ...

    @typing.overload
    def __getitem__(self, key: slice) -> tuple[BMVert, ...]:
        """

        :param key:
        :type key: slice
        :return:
        :rtype: tuple[BMVert, ...]
        """
        ...

    def __iter__(self) -> BMIter[BMVert]:
        """

        :return:
        :rtype: BMIter[BMVert]
        """
        ...

    def __len__(self) -> int:
        """

        :return:
        :rtype: int
        """
        ...

class BMesh:
    """The BMesh data structure"""

    edges: BMEdgeSeq
    """ This meshes edge sequence (read-only).

    :type: BMEdgeSeq
    """

    faces: BMFaceSeq
    """ This meshes face sequence (read-only).

    :type: BMFaceSeq
    """

    is_valid: bool
    """ True when this element is valid (hasn't been removed).

    :type: bool
    """

    is_wrapped: bool
    """ True when this mesh is owned by blender (typically the editmode BMesh).

    :type: bool
    """

    loops: BMLoopSeq
    """ This meshes loops (read-only).

    :type: BMLoopSeq
    """

    select_history: BMEditSelSeq
    """ Sequence of selected items (the last is displayed as active).

    :type: BMEditSelSeq
    """

    select_mode: set
    """ The selection mode, values can be {'VERT', 'EDGE', 'FACE'}, can't be assigned an empty set.

    :type: set
    """

    verts: BMVertSeq
    """ This meshes vert sequence (read-only).

    :type: BMVertSeq
    """

    def calc_loop_triangles(self):
        """Calculate triangle tessellation from quads/ngons.

        :return: The triangulated faces.
        """
        ...

    def calc_volume(self, signed: bool = False) -> float:
        """Calculate mesh volume based on face normals.

        :param signed: when signed is true, negative values may be returned.
        :type signed: bool
        :return: The volume of the mesh.
        :rtype: float
        """
        ...

    def clear(self):
        """Clear all mesh data."""
        ...

    def copy(self) -> typing_extensions.Self:
        """

        :return: A copy of this BMesh.
        :rtype: typing_extensions.Self
        """
        ...

    def free(self):
        """Explicitly free the BMesh data from memory, causing exceptions on further access."""
        ...

    def from_mesh(
        self,
        mesh: bpy.types.Mesh,
        face_normals: bool = True,
        vertex_normals: bool = True,
        use_shape_key: bool = False,
        shape_key_index: int = 0,
    ):
        """Initialize this bmesh from existing mesh datablock.

        :param mesh: The mesh data to load.
        :type mesh: bpy.types.Mesh
        :param face_normals:
        :type face_normals: bool
        :param vertex_normals:
        :type vertex_normals: bool
        :param use_shape_key: Use the locations from a shape key.
        :type use_shape_key: bool
        :param shape_key_index: The shape key index to use.
        :type shape_key_index: int
        """
        ...

    def from_object(
        self,
        object: bpy.types.Object,
        depsgraph: bpy.types.Depsgraph,
        cage: bool = False,
        face_normals: bool = True,
        vertex_normals: bool = True,
    ):
        """Initialize this bmesh from existing object data-block (only meshes are currently supported).

        :param object: The object data to load.
        :type object: bpy.types.Object
        :param depsgraph:
        :type depsgraph: bpy.types.Depsgraph
        :param cage: Get the mesh as a deformed cage.
        :type cage: bool
        :param face_normals: Calculate face normals.
        :type face_normals: bool
        :param vertex_normals: Calculate vertex normals.
        :type vertex_normals: bool
        """
        ...

    def normal_update(self):
        """Update normals of mesh faces and verts."""
        ...

    def select_flush(self, select: bool):
        """Flush selection, independent of the current selection mode.

        :param select: flush selection or de-selected elements.
        :type select: bool
        """
        ...

    def select_flush_mode(self):
        """flush selection based on the current mode current `BMesh.select_mode`."""
        ...

    def to_mesh(self, mesh: bpy.types.Mesh):
        """Writes this BMesh data into an existing Mesh datablock.

        :param mesh: The mesh data to write into.
        :type mesh: bpy.types.Mesh
        """
        ...

    def transform(
        self,
        matrix: collections.abc.Sequence[collections.abc.Sequence[float]]
        | mathutils.Matrix,
        filter: set = None,
    ):
        """Transform the mesh (optionally filtering flagged data only).

        :param matrix: transform matrix.
        :type matrix: collections.abc.Sequence[collections.abc.Sequence[float]] | mathutils.Matrix
        :param filter: set of values in ('SELECT', 'HIDE', 'SEAM', 'SMOOTH', 'TAG').
        :type filter: set
        """
        ...



import typing
import collections.abc
import typing_extensions

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

class AttributeGetterSetter:
    """Helper class to get and set attributes at an index for a domain."""

    ...

class SliceHelper:
    """Helper class to handle custom slicing."""

    ...

class GreasePencilStroke(AttributeGetterSetter):
    """A helper class to get access to stroke data."""

    aspect_ratio: typing.Any
    curve_type: typing.Any
    cyclic: typing.Any
    end_cap: typing.Any
    fill_color: typing.Any
    fill_opacity: typing.Any
    material_index: typing.Any
    points: typing.Any
    select: typing.Any
    softness: typing.Any
    start_cap: typing.Any
    time_start: typing.Any

    def add_points(self, count):
        """Add new points at the end of the stroke and returns the new points as a list.

        :param count:
        """
        ...

    def remove_points(self, count):
        """Remove points at the end of the stroke.

        :param count:
        """
        ...

class GreasePencilStrokePoint(AttributeGetterSetter):
    """A helper class to get access to stroke point data."""

    delta_time: typing.Any
    opacity: typing.Any
    position: typing.Any
    radius: typing.Any
    rotation: typing.Any
    select: typing.Any
    vertex_color: typing.Any

class GreasePencilStrokePointSlice(SliceHelper):
    """A helper class that represents a slice of GreasePencilStrokePoint's."""

    ...

class GreasePencilStrokeSlice(SliceHelper):
    """A helper class that represents a slice of GreasePencilStroke's."""

    ...

def DefAttributeGetterSetters(attributes_list):
    """A class decorator that reads a list of attribute information &
    creates properties on the class with getters & setters.

    """

    ...

def def_prop_for_attribute(attr_name, type, default, doc):
    """Creates a property that can read and write an attribute."""

    ...



"""
This module contains utility functions to handle custom previews.

It behaves as a high-level 'cached' previews manager.

This allows scripts to generate their own previews, and use them as icons in UI widgets
('icon_value' for UILayout functions).


--------------------

```__/__/__/scripts/templates_py/ui_previews_custom_icon.py```

"""

import typing
import collections.abc
import typing_extensions
import bpy.types

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

class ImagePreviewCollection:
    """Dictionary-like class of previews.This is a subclass of Python's built-in dict type,
    used to store multiple image previews.
    """

    def clear(self):
        """Clear all previews."""
        ...

    def close(self):
        """Close the collection and clear all previews."""
        ...

    def load(
        self,
        name: str | None,
        filepath: bytes | str | None,
        filetype: str | None,
        force_reload: bool | None = False,
    ) -> bpy.types.ImagePreview:
        """Generate a new preview from given file path.

        :param name: The name (unique id) identifying the preview.
        :type name: str | None
        :param filepath: The file path to generate the preview from.
        :type filepath: bytes | str | None
        :param filetype: The type of file, needed to generate the preview in ['IMAGE', 'MOVIE', 'BLEND', 'FONT'].
        :type filetype: str | None
        :param force_reload: If True, force running thumbnail manager even if preview already exists in cache.
        :type force_reload: bool | None
        :return: The Preview matching given name, or a new empty one.
        :rtype: bpy.types.ImagePreview
        """
        ...

    def new(self, name: str | None) -> bpy.types.ImagePreview:
        """Generate a new empty preview.

        :param name: The name (unique id) identifying the preview.
        :type name: str | None
        :return: The Preview matching given name, or a new empty one.
        :rtype: bpy.types.ImagePreview
        """
        ...

def new() -> ImagePreviewCollection:
    """

    :return: a new preview collection.
    :rtype: ImagePreviewCollection
    """

    ...

def remove(pcoll: ImagePreviewCollection | None):
    """Remove the specified previews collection.

    :param pcoll: Preview collection to close.
    :type pcoll: ImagePreviewCollection | None
    """

    ...



import typing
import collections.abc
import typing_extensions

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

def check(check_ctxt, msgs, key, msgsrc, settings):
    """Performs a set of checks over the given key (context, message)..."""

    ...

def dump_addon_bl_info(msgs, reports, module, settings): ...
def dump_addon_messages(addon_module_name, do_checks, settings): ...
def dump_asset_messages(msgs, reports, settings): ...
def dump_extension_metadata(msgs, reports, settings): ...
def dump_messages(do_messages, do_checks, settings): ...
def dump_preset_messages(msgs, reports, settings): ...
def dump_py_messages(msgs, reports, addons, settings, addons_only=False): ...
def dump_py_messages_from_files(msgs, reports, files, settings): ...
def dump_rna_messages(msgs, reports, settings, verbose=False):
    """Dump into messages dict all RNA-defined UI messages (labels en tooltips)."""

    ...

def dump_src_messages(msgs, reports, settings): ...
def dump_template_messages(msgs, reports, settings): ...
def init_spell_check(settings, lang="en_US"): ...
def main(): ...
def print_info(reports, pot): ...
def process_msg(msgs, msgctxt, msgid, msgsrc, reports, check_ctxt, settings): ...



import typing
import collections.abc
import typing_extensions

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

def keyconfig_data_oskey_from_ctrl(keyconfig_data_src, *, filter_fn=None): ...
def keyconfig_data_oskey_from_ctrl_for_macos(keyconfig_data_src):
    """Use for apple since Cmd is typically used in-place of Ctrl."""

    ...



import typing
import collections.abc
import typing_extensions

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

def complete(line: str) -> list:
    """Returns a list containing the completion possibilities for an import line.

        :param line: incomplete line which contains an import statement:

    import xml.d
    from xml.dom import
        :type line: str
        :return: list of completion possibilities
        :rtype: list
    """

    ...

def get_root_modules() -> list:
    """Returns a list containing the names of all the modules available in the
    folders of the python-path.

        :return: modules
        :rtype: list
    """

    ...

def module_list(path: str) -> list:
    """Return the list containing the names of the modules available in
    the given folder.

        :param path: folder path
        :type path: str
        :return: modules
        :rtype: list
    """

    ...



import typing
import collections.abc
import typing_extensions

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

class _TempModuleOverride:
    module: typing.Any
    module_name: typing.Any
    module_override: typing.Any

def add_scrollback(text, text_type): ...
def autocomplete(context): ...
def banner(context): ...
def copy_as_script(context): ...
def execute(context, is_interactive): ...
def get_console(console_id):
    """helper function for console operators
    currently each text data block gets its own
    console - code.InteractiveConsole()
    ...which is stored in this function.console_id can be any hashable type

    """

    ...

def replace_help(namespace): ...



"""
This module provides access to the GPU capabilities.

"""

import typing
import collections.abc
import typing_extensions

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

def compute_shader_support_get() -> bool:
    """Are compute shaders supported.

    :return: True when supported, False when not supported.
    :rtype: bool
    """

    ...

def extensions_get():
    """Get supported extensions in the current context.

    :return: Extensions.
    """

    ...

def hdr_support_get() -> bool:
    """Return whether GPU backend supports High Dynamic range for viewport.

    :return: HDR support available.
    :rtype: bool
    """

    ...

def max_batch_indices_get() -> int:
    """Get maximum number of vertex array indices.

    :return: Number of indices.
    :rtype: int
    """

    ...

def max_batch_vertices_get() -> int:
    """Get maximum number of vertex array vertices.

    :return: Number of vertices.
    :rtype: int
    """

    ...

def max_images_get() -> int:
    """Get maximum supported number of image units.

    :return: Number of image units.
    :rtype: int
    """

    ...

def max_texture_layers_get() -> int:
    """Get maximum number of layers in texture.

    :return: Number of layers.
    :rtype: int
    """

    ...

def max_texture_size_get() -> int:
    """Get estimated maximum texture size to be able to handle.

    :return: Texture size.
    :rtype: int
    """

    ...

def max_textures_frag_get() -> int:
    """Get maximum supported texture image units used for
    accessing texture maps from the fragment shader.

        :return: Texture image units.
        :rtype: int
    """

    ...

def max_textures_geom_get() -> int:
    """Get maximum supported texture image units used for
    accessing texture maps from the geometry shader.

        :return: Texture image units.
        :rtype: int
    """

    ...

def max_textures_get() -> int:
    """Get maximum supported texture image units used for
    accessing texture maps from the vertex shader and the
    fragment processor.

        :return: Texture image units.
        :rtype: int
    """

    ...

def max_textures_vert_get() -> int:
    """Get maximum supported texture image units used for
    accessing texture maps from the vertex shader.

        :return: Texture image units.
        :rtype: int
    """

    ...

def max_uniforms_frag_get() -> int:
    """Get maximum number of values held in uniform variable
    storage for a fragment shader.

        :return: Number of values.
        :rtype: int
    """

    ...

def max_uniforms_vert_get() -> int:
    """Get maximum number of values held in uniform variable
    storage for a vertex shader.

        :return: Number of values.
        :rtype: int
    """

    ...

def max_varying_floats_get() -> int:
    """Get maximum number of varying variables used by
    vertex and fragment shaders.

        :return: Number of variables.
        :rtype: int
    """

    ...

def max_vertex_attribs_get() -> int:
    """Get maximum number of vertex attributes accessible to
    a vertex shader.

        :return: Number of attributes.
        :rtype: int
    """

    ...

def max_work_group_count_get(index: int) -> int:
    """Get maximum number of work groups that may be dispatched to a compute shader.

    :param index: Index of the dimension.
    :type index: int
    :return: Maximum number of work groups for the queried dimension.
    :rtype: int
    """

    ...

def max_work_group_size_get(index: int) -> int:
    """Get maximum size of a work group that may be dispatched to a compute shader.

    :param index: Index of the dimension.
    :type index: int
    :return: Maximum size of a work group for the queried dimension.
    :rtype: int
    """

    ...

def shader_image_load_store_support_get() -> bool:
    """Is image load/store supported.

    :return: True when supported, False when not supported.
    :rtype: bool
    """

    ...



import typing
import collections.abc
import typing_extensions
import bpy.types

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

class ANIM_OT_keying_set_export(bpy.types.Operator):
    """Export Keying Set to a Python script"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

    def invoke(self, context, _event):
        """

        :param context:
        :param _event:
        """
        ...

class ARMATURE_OT_collection_remove_unused(bpy.types.Operator):
    """Remove all bone collections that have neither bones nor children. This is done recursively, so bone collections that only have unused children are also removed"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

    def execute_edit_mode(self, context):
        """

        :param context:
        """
        ...

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """
        ...

    def remove_bcolls(self, armature, bcolls_to_remove):
        """

        :param armature:
        :param bcolls_to_remove:
        """
        ...

    def visit(self, bcoll, bcolls_with_bones, bcolls_to_remove):
        """

        :param bcoll:
        :param bcolls_with_bones:
        :param bcolls_to_remove:
        """
        ...

class ARMATURE_OT_collection_show_all(bpy.types.Operator):
    """Show all bone collections"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """
        ...

class ARMATURE_OT_collection_unsolo_all(bpy.types.Operator):
    """Clear the 'solo' setting on all bone collections"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """
        ...

class ARMATURE_OT_copy_bone_color_to_selected(bpy.types.Operator):
    """Copy the bone color of the active bone to all selected bones"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """
        ...

class ClearUselessActions(bpy.types.Operator):
    """Mark actions with no F-Curves for deletion after save and reload of file preserving "action libraries" """

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, _context):
        """

        :param _context:
        """
        ...

    @classmethod
    def poll(cls, _context):
        """

        :param _context:
        """
        ...

class NLA_OT_bake(bpy.types.Operator):
    """Bake all selected objects location/scale/rotation animation to an action"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

    def invoke(self, context, _event):
        """

        :param context:
        :param _event:
        """
        ...

class UpdateAnimatedTransformConstraint(bpy.types.Operator):
    """Update f-curves/drivers affecting Transform constraints (use it with files from 2.70 and earlier)"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...



import typing
import collections.abc
import typing_extensions
import bpy.types

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

def rule_add(
    override_context: bpy.types.Context | dict[str, typing.Any] = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
    *,
    type: typing.Literal[
        "GOAL",
        "AVOID",
        "AVOID_COLLISION",
        "SEPARATE",
        "FLOCK",
        "FOLLOW_LEADER",
        "AVERAGE_SPEED",
        "FIGHT",
    ]
    | None = "GOAL",
):
    """Add a boid rule to the current boid state

    :type override_context: bpy.types.Context | dict[str, typing.Any]
    :type execution_context: int | str | None
    :type undo: bool | None
    :param type: Type
    :type type: typing.Literal['GOAL', 'AVOID', 'AVOID_COLLISION', 'SEPARATE', 'FLOCK', 'FOLLOW_LEADER', 'AVERAGE_SPEED', 'FIGHT'] | None
    """

    ...

def rule_del(
    override_context: bpy.types.Context | dict[str, typing.Any] = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Delete current boid rule

    :type override_context: bpy.types.Context | dict[str, typing.Any]
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def rule_move_down(
    override_context: bpy.types.Context | dict[str, typing.Any] = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Move boid rule down in the list

    :type override_context: bpy.types.Context | dict[str, typing.Any]
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def rule_move_up(
    override_context: bpy.types.Context | dict[str, typing.Any] = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Move boid rule up in the list

    :type override_context: bpy.types.Context | dict[str, typing.Any]
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def state_add(
    override_context: bpy.types.Context | dict[str, typing.Any] = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Add a boid state to the particle system

    :type override_context: bpy.types.Context | dict[str, typing.Any]
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def state_del(
    override_context: bpy.types.Context | dict[str, typing.Any] = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Delete current boid state

    :type override_context: bpy.types.Context | dict[str, typing.Any]
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def state_move_down(
    override_context: bpy.types.Context | dict[str, typing.Any] = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Move boid state down in the list

    :type override_context: bpy.types.Context | dict[str, typing.Any]
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def state_move_up(
    override_context: bpy.types.Context | dict[str, typing.Any] = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Move boid state up in the list

    :type override_context: bpy.types.Context | dict[str, typing.Any]
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...



import typing
import collections.abc
import typing_extensions

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

def do_clear_previews(do_objects, do_collections, do_scenes, do_data_intern): ...
def do_previews(do_objects, do_collections, do_scenes, do_data_intern): ...
def ids_nolib(bids): ...
def ids_nolib_with_preview(bids): ...
def main(): ...
def rna_backup_gen(data, include_props=None, exclude_props=None, root=()): ...
def rna_backup_restore(data, backup): ...



import typing
import collections.abc
import typing_extensions
import bpy.types

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

class OUTLINER_HT_header(bpy.types.Header):
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, context):
        """

        :param context:
        """
        ...

class OUTLINER_MT_asset(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, _context):
        """

        :param _context:
        """
        ...

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """
        ...

class OUTLINER_MT_collection(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, context):
        """

        :param context:
        """
        ...

class OUTLINER_MT_collection_new(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, context):
        """

        :param context:
        """
        ...

    @staticmethod
    def draw_without_context_menu(_context, layout):
        """

        :param _context:
        :param layout:
        """
        ...

class OUTLINER_MT_collection_view_layer(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, context):
        """

        :param context:
        """
        ...

class OUTLINER_MT_collection_visibility(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, _context):
        """

        :param _context:
        """
        ...

class OUTLINER_MT_context_menu(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, context):
        """

        :param context:
        """
        ...

    @staticmethod
    def draw_common_operators(layout):
        """

        :param layout:
        """
        ...

class OUTLINER_MT_context_menu_view(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, _context):
        """

        :param _context:
        """
        ...

class OUTLINER_MT_edit_datablocks(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, _context):
        """

        :param _context:
        """
        ...

class OUTLINER_MT_editor_menus(bpy.types.Menu):
    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, context):
        """

        :param context:
        """
        ...

class OUTLINER_MT_liboverride(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, _context):
        """

        :param _context:
        """
        ...

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """
        ...

class OUTLINER_MT_object(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, context):
        """

        :param context:
        """
        ...

class OUTLINER_MT_view_pie(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, _context):
        """

        :param _context:
        """
        ...

class OUTLINER_PT_filter(bpy.types.Panel):
    bl_label: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def draw(self, context):
        """

        :param context:
        """
        ...

def has_selected_ids_in_context(context): ...
